unit WddxApi;

{
  Class             : TWddxApi
  Author            : Michael Pham (mpham5@mediaone.net)
  Created           : 04/06/2001
  Unit Version      : 1.0.1

  DESCRIPTION: TWddxApi is a class that encapsulates the WDDX SDK COM Library. It is designed specifically
  handle some Delphi types and convert them into a suitable WDDX XML Packet to send to another
  machine that can deserialize and read the data, such as a ColdFusion Server or any application
  that uses the WDDX SDK. This is released under the GNU GPL. You may redistribute, improve upon
  and use royalty-free in your applications.

  Can serialize any TDataSet or TStrings descendant.
  Can also serialize an array of OleVariants.
  Returns a TDataSet as a WDDXRecordset.
  Returns a TStrings as a WDDXStructure.
  When calling Deserialize(), you must typecast the OleVariant to the expected class or type.

  Maybe in the next release, I will provide a function to return a TDataSet from Deserialize().
  You may call StructToStrings() and StringsToStruct() to convert between to two.
}

interface

uses
  Windows, SysUtils, Classes, ComObj, WDDX_COMLib_TLB, Db;

type
  TWddxApi = class(TComponent)
  private
    { Private declarations }
    FDeserializer: WDDXDeserializer;
    FJSConverter: WDDXJSConverter;
    FSerializer: WDDXSerializer;
  protected
    { Protected properties }
    property Deserializer: WDDXDeserializer read FDeserializer;
    property Serializer: WDDXSerializer read FSerializer;
    property JSConverter: WDDXJSConverter read FJSConverter;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Deserialize(Packet: String): OleVariant; overload; virtual;
    function Serialize(Data: OleVariant): TStrings; overload; virtual;
    function Serialize(DataSet: TDataSet): TStrings; overload; virtual;
    function Serialize(Strings: TStrings): TStrings; overload; virtual;
    function Serialize(VarArray: array of OleVariant): TStrings; overload; virtual;
    function WddxToScript(Packet: TStrings; TopLevelObject: String): TStrings; overload; virtual;
    function WddxToScript(Packet, TopLevelObject: String): TStrings; overload; virtual;
    function WddxToScript(Packet: WDDXStruct; TopLevelObject: String): TStrings; overload; virtual;
    function NewStructure: WDDXStruct;
    function NewRecordset: WDDXRecordset;
  published
    { Published properties }
  end;

  procedure StructToStrings(const Struct: IWDDXStruct; var Strings: TStrings);
  function StringsToStruct(const Strings: TStrings): IWDDXStruct;

  procedure Register;

implementation

  procedure Register;
  begin
    RegisterComponents('WDDX', [TWddxApi]);
  end;

  procedure StructToStrings(const Struct: IWDDXStruct; var Strings: TStrings);
  var
    ar: array of String;
    i: Integer;
  begin
    Strings.Clear;
    ar := Struct.getPropNames;
    for i:=0 to Length(ar) do
      Strings.Add(ar[i]+'='+Struct.getProp(ar[i]));
  end;

  function StringsToStruct(const Strings: TStrings): IWDDXStruct;
  var
    i: Integer;
  begin
    Result := CoWDDXStruct.Create;
    for i:=0 to Strings.Count-1 do
      Result.setProp(Strings.Names[i], Strings.Values[Strings.Names[i]]);
  end;

{ TWddxApi }

constructor TWddxApi.Create(AOwner: TComponent);
begin
  inherited;
  FDeserializer   := CoWDDXDeserializer.Create;
  FSerializer     := CoWDDXSerializer.Create;
  FJSConverter    := CoWDDXJSConverter.Create;
end;

destructor TWddxApi.Destroy;
begin
  FDeserializer   := nil;
  FSerializer     := nil;
  FJSConverter    := nil;
  inherited;
end;

function TWddxApi.Deserialize(Packet: String): OleVariant;
begin
  Result := FDeserializer.deserialize(Packet);
end;

function TWddxApi.Serialize(DataSet: TDataSet): TStrings;
var
  i: Integer;
  s: WDDXRecordset;
begin
  s := NewRecordset;
  DataSet.First;
  // create columns
  for i:=0 to DataSet.FieldCount-1 do
    s.addColumn(DataSet.Fields[i].DisplayName);
  // create rows
  s.addRows(DataSet.RecordCount);
  while not DataSet.Eof do
  begin
    // assign data values
    for i:=0 to s.getColumnCount-1 do
      s.setField(DataSet.RecNo, DataSet.Fields[i].DisplayName,
        DataSet.Fields[i].Value);
    DataSet.Next;
  end;
  Result := Serialize(s);
end;

function TWddxApi.Serialize(Strings: TStrings): TStrings;
begin
  Result := Serialize(StringsToStruct(Strings));
end;

function TWddxApi.Serialize(Data: OleVariant): TStrings;
begin
  Result := TStringList.Create;
  Result.Text := FSerializer.serialize(Data);
end;

function TWddxApi.Serialize(VarArray: array of OleVariant): TStrings;
begin
  Result := Serialize(VarArray);
end;

function TWddxApi.WddxToScript(Packet: TStrings; TopLevelObject: String): TStrings;
begin
  Result := WddxToScript(Packet.Text, TopLevelObject);
end;

function TWddxApi.WddxToScript(Packet, TopLevelObject: String): TStrings;
begin
  Result := TStringList.Create;
  Result.Text := FJSConverter.convertWddx(Packet, TopLevelObject);
end;

function TWddxApi.WddxToScript(Packet: WDDXStruct; TopLevelObject: String): TStrings;
begin
  Result := TStringList.Create;
  Result.Text := FJSConverter.convertData(Packet, TopLevelObject);
end;

function TWddxApi.NewStructure: WDDXStruct;
begin
  Result := CoWDDXStruct.Create;
end;

function TWddxApi.NewRecordset: WDDXRecordset;
begin
  Result := CoWDDXRecordset.Create;
end;

end.
